@@ -1,4 +1,5 @@
-/// Types produced by the _ChRIS_ backend (CUBE) API.
+//! Types produced by the _ChRIS_ backend (CUBE) API.
+
 use crate::common_types::Username;
 use aliri_braid::braid;
 use serde::{Deserialize, Serialize};
@@ -16,15 +17,20 @@ pub struct FeedsPaginatedUrl;
 
 #[derive(Debug, Deserialize)]
 pub struct CUBELinks {
-    pub files: AllFilesUrl,
+    pub files: FeedFilesUrl,
     pub uploadedfiles: UploadedFilesUrl,
     pub user: UserUrl,
     pub pipelines: PipelinesUrl,
+    pub filebrowser: FileBrowserUrl,
 }
 
+/// CUBE file browser API URL, e.g. `https://cube.chrisproject.org/api/v1/filebrowser/`
+#[braid(serde)]
+pub struct FileBrowserUrl;
+
 /// CUBE files resource URL, e.g. `https://cube.chrisproject.org/api/v1/files/`
 #[braid(serde)]
-pub struct AllFilesUrl;
+pub struct FeedFilesUrl;
 
 /// CUBE uploaded files resource URL, e.g. `https://cube.chrisproject.org/api/v1/uploadedfiles/`
 #[braid(serde)]
@@ -81,6 +87,52 @@ pub struct PipelineInstancesUrl;
 #[braid(serde)]
 pub struct PipelineWorkflowsUrl;
 
+/// A URL which produces a collection of files.
+///
+/// # Examples
+///
+/// - `https://cube.chrisproject.org/api/v1/files/`
+/// - `https://cube.chrisproject.org/api/v1/files/search/`
+/// - `https://cube.chrisproject.org/api/v1/uploadedfiles/search/?fname=txt`
+/// - `https://cube.chrisproject.org/api/v1/20/files/`
+/// - `https://cube.chrisproject.org/api/v1/plugins/instances/40/files/`
+#[braid(serde)]
+pub struct AnyFilesUrl;
+
+/// Download URL for a file resource.
+///
+/// # Examples
+///
+/// - `https://cube.chrisproject.org/api/v1/files/84360/aparc.a2009s+aseg.mgz`
+#[braid(serde)]
+pub struct FileResourceUrl;
+
+/// File fname.
+#[braid(serde)]
+pub struct FileResourceFname;
+
+/// A CUBE item which has a `file_resource` and `fname`.
+pub trait Downloadable {
+    fn file_resource(&self) -> &FileResourceUrl;
+    fn fname(&self) -> &FileResourceFname;
+}
+
+#[derive(Deserialize)]
+pub struct DownloadableFile {
+    file_resource: FileResourceUrl,
+    fname: FileResourceFname,
+}
+
+impl Downloadable for DownloadableFile {
+    fn file_resource(&self) -> &FileResourceUrl {
+        &self.file_resource
+    }
+
+    fn fname(&self) -> &FileResourceFname {
+        &self.fname
+    }
+}
+
 #[derive(Debug, Deserialize)]
 pub struct PipelineUploadResponse {
     pub url: PipelineUrl,
@@ -105,8 +157,18 @@ pub struct FileUploadResponse {
     pub url: String,
     pub id: u32,
     pub creation_date: String,
-    pub fname: String,
+    fname: FileResourceFname,
     pub fsize: u32,
-    pub file_resource: String,
+    file_resource: FileResourceUrl,
     pub owner: String,
 }
+
+impl Downloadable for FileUploadResponse {
+    fn file_resource(&self) -> &FileResourceUrl {
+        &self.file_resource
+    }
+
+    fn fname(&self) -> &FileResourceFname {
+        &self.fname
+    }
+}
